import board # Library to access board pins
import busio # Library to access i2c bus
import time # Library for delays
import digitalio # Library to configure digital pins
import analogio # Library to configure analog pins
import pulseio # Library to control pwm
import adafruit_tlv493d # Library to control TLV493D sensor
from adafruit_motor import servo # Library for servos control

# i2c object declaration
i2c = busio.I2C(board.SCL, board.SDA)

# tlv493d object declaration
tlv = adafruit_tlv493d.TLV493D(i2c)

# Servos objects declaration
pwm1 = pulseio.PWMOut(board.D2, frequency=50)
pwm2 = pulseio.PWMOut(board.D5, frequency=50)
pwm3 = pulseio.PWMOut(board.D8, frequency=50)
pwm4 = pulseio.PWMOut(board.D9, frequency=50)

# Servos objects declaration
servo1 = servo.Servo(pwm1)#, min_pulse=750, max_pulse=2250)
servo2 = servo.Servo(pwm2)#, min_pulse=750, max_pulse=2250)
servo3 = servo.Servo(pwm3)#, min_pulse=750, max_pulse=2250)
servo4 = servo.Servo(pwm4)#, min_pulse=750, max_pulse=2250)

# Switch pin declaration
switch = digitalio.DigitalInOut(board.D4)
switch.direction = digitalio.Direction.INPUT
switch.pull = digitalio.Pull.UP

# Gripper pin declaration
gripper = digitalio.DigitalInOut(board.D3)
gripper.direction = digitalio.Direction.INPUT
gripper.pull = digitalio.Pull.UP

# Led declaration
led = digitalio.DigitalInOut(board.LED)
led.direction = digitalio.Direction.OUTPUT

# Range for constrain the joystick coordinates
limit = 100
# Whether or not to control the arm
armIsActive = False
# Previous switch state
lastSwitchState = False
# Whether or not to control the gripper
gripperIsActive = False
# Previous gripper state
lastGripperState = False

# Servos initialization
servo1.angle = 0
servo2.angle = 0
servo3.angle = 30
servo4.angle = 0

# Variable to save switch state
switchState = False
# Vaiable to save gripper state
gripperState = False

# FUNCTIONS

# Scale a value from one range to another
def _map(val,fromLow,fromHigh,toLow,toHigh):
    return int((x-fromLow)*(toHigh-toLow)/(fromHigh-fromLow)+toLow)

# Steps
def steps(axis):
    return round((axis - pot_min) / step)

# Constrain a value to a range
def constrain(val,min_val,max_val):
#    if(val<min_val):
#        return min_val
#    elif(val>max_val):
#        return max_val
#    else:
#        return val
    return min(max_val, max(min_val,val))

while True:

    # Turn OFF Led
    led.value = True

    # Read the switch
    switchState = switch.value
    # Activate the arm
    if (switchState != lastSwitchState):
        if (switchState == True):
            armIsActive = not(armIsActive)
    # Save switch state for next comparison
    lastSwitchState = switchState

    # Check if arm is active
    if armIsActive:
        # Turn ON Led
        led.value = False

        # Read and constrain the two axles
        x = constrain(-int(tlv.magnetic[1]),-limit,limit)
        y = constrain(int(tlv.magnetic[0]),-limit,limit)

        # Scale the two axles
        x = _map(x,-limit,limit,30,150)
        y = _map(y,-limit,limit,1,179)

        #x = constrain(x,0,180)
        #y = constrain(y,0,180)

        # Print coordinates
        print("X = "+str(x))
        print("Y = "+str(y))

        # Move the arm
        servo2.angle = y 
        servo3.angle = x

        # Read gripper switch
        gripperState = gripper.value
        # Open or close the gripper
        if (gripperState != lastGripperState)and(gripperState == False):
            gripperIsActive = not(gripperIsActive)
            if(gripperIsActive == False):
                servo4.angle = 180
            else:
                servo4.angle = 0
        # Save gripper switch for next comparison
        lastGripperState = gripperState
'''
import board
import pulseio
         
for pin_name in dir(board):
    pin = getattr(board, pin_name)
    try:
        p = pulseio.PWMOut(pin)
        p.deinit()
        print("PWM on:", pin_name)  # Prints the valid, PWM-capable pins!                                                                   
    except ValueError:  # This is the error returned when the pin is invalid.
        print("No PWM on:", pin_name)  # Prints the invalid pins.
    except RuntimeError:  # Timer conflict error.
        print("Timers in use:", pin_name)  # Prints the timer conflict pins.
    except TypeError:  # Error returned when checking a non-pin object in dir(board).
        pass  # Passes over non-pin objects in dir(board).
'''
